﻿/***************************************************************************************************************************
Create Book Cover with Page Control plug-in from DTP Tools
Version 1.0

This script only works as a guide for you and you should always check the spine size settings with your provider.

We've intentionally left this script open for you to edit and tweak to match your providers paper specs. 
We'll be happy to see your improvements and with your consent we'll share them with other users though Page Control page.
Please do not remove association to our product.

****************************************************************************************************************************/
main();


function main() {
	var scriptName = "BookCover";

	with(app) {

		try {
			var myDocument = app.activeDocument;
		} catch (e) {
				alert("No active document. Please open a document.");
		
				exit(-1);
		}
	
		var myPref_dpi = myDocument.viewPreferences.pointsPerInch;
		var myPref_horizontalMeasurementUnits = myDocument.viewPreferences.horizontalMeasurementUnits;
		var myPref_verticalMeasurementUnits = myDocument.viewPreferences.verticalMeasurementUnits;
		var myPref_numberOfPages = myDocument.documentPreferences.pagesPerDocument;
		var myPref_pageHeight = myDocument.documentPreferences.pageHeight;
		var myPref_pageWidth = myDocument.documentPreferences.pageWidth;
		var myPref_numberOfBooks = app.books.count();;
		
		var bookCoverDialog = dialogs.add();
		
		bookCoverDialog.name ="Book Cover Script";
		
		with (bookCoverDialog.dialogColumns.add()) {
			
			//use custom page count
			with(borderPanels.add()) {
				with(dialogColumns.add()) {
			with(dialogRows.add()) {
				
				var myWG_useRadioGroup = radiobuttonGroups.add();
				var myWG_useCustomPageCount = myWG_useRadioGroup.radiobuttonControls.add({staticLabel: "Use custom page count ",
					checkedState:false});
				
				var myWC_customPageCount = integerEditboxes.add();
				
				staticTexts.add({staticLabel: " pages "});
				
				}
			
			//use document page count
			with(dialogRows.add()) {
				var myWG_useDocumentPageCount = myWG_useRadioGroup.radiobuttonControls.add({staticLabel: "Use current document page count: "+myPref_numberOfPages,
					checkedState:true, minWidth: 160});
				}
			
			//use selected book
			var myWG_useBookPageCount;
			if(myPref_numberOfBooks > 0) {
				with(dialogRows.add()) {
				myWG_useBookPageCount = myWG_useRadioGroup.radiobuttonControls.add({staticLabel: "Use selected book page count",
					checkedState:false, minWidth: 160});
				}
			} else {
				myWG_useBookPageCount = false;
			}
		}}
				
			//paper weight
			with(borderPanels.add()) {
				with(dialogColumns.add()) {
			
			with(dialogRows.add()) {
				staticTexts.add({staticLabel: "Paper weight"});
				
				}
			
			with(dialogRows.add()) {
				staticTexts.add({staticLabel: "", minWidth: 360});
				
				}
			with(dialogRows.add()) {
				with(radiobuttonGroups.add()) {
					var myWG_offsetPaper = radiobuttonControls.add({staticLabel: "Offset paper:",
					checkedState:true, minWidth: 160});
				var myWG_gmsPaper = radiobuttonControls.add({staticLabel: "Metric system:",
					checkedState:false });
				}
				
			}
			with(dialogRows.add()) {
				with(dialogRows.add()) {
					//with(borderPanels.add()) {
						staticTexts.add({staticLabel: "Offset #:"});
						var myWG_paperWeight = integerComboboxes.add({editValue:40, minimumValue:40, maximumValue:267, largeNudge:5, smallNudge:1, 
							stringList: [ "40","45","50","60","70","73","81","90","100","110","120","135","137","146","165","170","183","192","208","220","267"]});
					//}
				}
				with(dialogRows.add()) {
					//with(borderPanels.add()) {
						staticTexts.add({staticLabel: "Metric:"});
						var myWG_paperWeightGMS = integerComboboxes.add({editValue:60, minimumValue:60, maximumValue:400, largeNudge:5, smallNudge:1, 
							stringList: [ "60","80","100","120","150","170","190","200","250","300","350"]});
						staticTexts.add({staticLabel: "gsm"});
					//}
				}	
			}
		}}
			
			// modify spine
			with(dialogRows.add()) {
				var modifySpineEnablingGroup = enablingGroups.add({staticLabel:"Modify spine", checkedState: false});
				
				with(modifySpineEnablingGroup) {
					staticTexts.add({staticLabel: "Modify spine "});
					var myWG_modifySpineWeight = measurementComboboxes.add({stringList: [ "-5","-4", "-3",  "-2","-1","-0.75","-0.50","-0.25", "0",  "0.25", "0.50", "0.75", "1", "2", "3", "4", "5"],
						editContents: "0", editUnits:myPref_horizontalMeasurementUnits, minWidth:90});
					}
				
				}
			
			//flaps
			with(dialogRows.add()) {
				// flaps
				staticTexts.add({staticLabel: " Flaps (if applicable): "});
				
				var myWG_flapsSize = measurementEditboxes.add({editContents: "0", editUnits:myPref_horizontalMeasurementUnits, minWidth:80});
		
		
			}
		}
	}

	with (bookCoverDialog.dialogColumns.add()) {
		with(dialogRows.add()) {
			with(borderPanels.add()) {
				var myWG_booksGroup = enablingGroups.add({staticLabel:"Books",checkedState:false});
				with (myWG_booksGroup) {
			
					var books = app.books;
					var myWG_books = dropdowns.add({minWidth: 130});
				
					var bookArray = new Array();
					for(var i= books.count()-1; i >=0;i--) {
						bookArray.push(books[i].name);
						}
					if (books.count()!=0) {
						
						myWG_booksGroup.checkedState = true;
						
					} else {
						bookArray.push("<<No book is open>>");
					}
					myWG_books.stringList = bookArray;
					myWG_books.selectedIndex =  0;
				}
			}
		}
	
		//covers - soft, soft with flaps, hard, hard with flaps
		with(dialogRows.add()) {
			with(borderPanels.add()){
				
					var myWG_coverGroup = radiobuttonGroups.add();
				
					//0
					var myWG_softCover = myWG_coverGroup.radiobuttonControls.add({staticLabel: "soft cover  ",
						checkedState:true, minWidth: 160});
					
					//1
					var myWG_softCoverFlaps = myWG_coverGroup.radiobuttonControls.add({staticLabel: "soft cover  with flaps",
						checkedState:false, minWidth: 160});
				
					//2
					var myWG_hardCover = myWG_coverGroup.radiobuttonControls.add({staticLabel: "hard cover ",
						checkedState:false, minWidth: 160});
				
					//3
					var myWG_hardCoverFlaps = myWG_coverGroup.radiobuttonControls.add({staticLabel: "dust jacket with flaps",
						checkedState:false, minWidth: 160});
				
					}
			}
		
		//page rounding
		with(dialogRows.add()) {
			with(borderPanels.add()){
				
					var myWG_bookletGroup = radiobuttonGroups.add();
				
					//0
					var myWG_noRound = myWG_bookletGroup.radiobuttonControls.add({staticLabel: "Use exact number of pages",
						checkedState:true, minWidth: 160});
					
					//1
					var myWG_8Round = myWG_bookletGroup.radiobuttonControls.add({staticLabel: "Round to 8 page booklet",
						checkedState:false, minWidth: 160});
				
					//2
					var myWG_16Round = myWG_bookletGroup.radiobuttonControls.add({staticLabel: "Round to 16 page booklet",
						checkedState:false, minWidth: 160});
					}
			}
	}

	//show dialog
	if(!bookCoverDialog.show()) {
		bookCoverDialog.destroy();
		
		exit(1);
		}
	
	//vars
	var numberOfPages;
	var paperWeight;
	var flaps;
	var modifiedSize;
	var cover;
	
	//number of pages
	if (myWG_useCustomPageCount.checkedState) {
		numberOfPages = myWC_customPageCount.editValue;
	} 
	
	if (myWG_useDocumentPageCount.checkedState) {
		numberOfPages  = myPref_numberOfPages;
	}
		
	if(myWG_useBookPageCount !=false) {
		if (myWG_useBookPageCount.checkedState) {
			
			//get selected book
			var bookName = myWG_books.stringList[myWG_books.selectedIndex];
			var book = app.books.itemByName (bookName);
			
			//get its contents
			var bookContents = book.bookContents;
			
			//get range of the last content
			var documentPageRange = bookContents[bookContents.length-1].documentPageRange;
			
			//get array of strings (first and last page of the last content)
			var pages = documentPageRange.split("-",2);
			
			//get the last page of the last content
			numberOfPages = pages[pages.length-1];
		}
	}

	//round and round we go
	if(!myWG_noRound.checkedState) {
		if (myWG_8Round.checkedState) {
			numberOfPages = roundPagesToXPageBooklet(numberOfPages, 8);
		}
	
		if (myWG_16Round.checkedState) {
			numberOfPages = roundPagesToXPageBooklet(numberOfPages, 16);
		}
	}
	
	//paper weight
	paperWeight = myWG_paperWeight.editValue;
		
	//cover
	if (myWG_softCover.checkedState) {
		cover =0;
		}
	
	if (myWG_softCoverFlaps.checkedState) {
		cover =1;
		}
	if (myWG_hardCover.checkedState) {
		cover =2;
		}
	
	if (myWG_hardCoverFlaps.checkedState) {
		cover =3;
		}
	
	//modified size
	if (modifySpineEnablingGroup.checkedState) {
		//transfer to mm
		modifiedSize = 25.4*convertUnitsToPoints(myPref_horizontalMeasurementUnits,  myWG_modifySpineWeight.editContents,
		myDocument.viewPreferences.pointsPerInch)/myDocument.viewPreferences.pointsPerInch;
		} else {
		modifiedSize = 0;	
	}
	
	//spine width
	var spineWidth = calculateSpineWidth(numberOfPages,paperWeight,modifiedSize);
	
	//if metric system is checked, use metric system
	if (myWG_gmsPaper.checkedState) {
		paperWeight = myWG_paperWeightGMS.editValue;
		spineWidth = calculateSpineWidthAccordingToGMS(numberOfPages,paperWeight,modifiedSize);
		
	}
	
	//flaps
	flaps = myWG_flapsSize.editContents;

	//creating cover
	createCover(myPref_horizontalMeasurementUnits, myPref_verticalMeasurementUnits,
	myPref_pageHeight,myPref_pageWidth,spineWidth,flaps,cover);
	
	//destroying dialog
	bookCoverDialog.destroy();
		
	exit(0);
	
}

//function for calculating out spine according to metric system
function calculateSpineWidthAccordingToGMS(numberOfPages,gms, modifiedSize) {
	
	var multiplier = 0.00124*gms + 0.00569;

	var width = multiplier *numberOfPages/2;

	width+= modifiedSize;
	
	return width;
}


//rounds pages up to the next integral multiple of a number defined by xPageBooklet
function roundPagesToXPageBooklet(pages, xPageBooklet) {
	var newNumberOfPages;
	
	var mult = pages/xPageBooklet;
	
	newNumberOfPages = Math.ceil(mult) * xPageBooklet;
	
	return newNumberOfPages;
}

//calculates spine width from number of pages, paper width and modified size
function calculateSpineWidth(numberOfPages,paperWeight, modifiedSize) {
	var width;
	var multiplier = 0;
	switch(paperWeight) {
		case 40:
		multiplier = 0.081;
		break;
		case 45:
		multiplier = 0.092;
		break;
		case 50:
		multiplier = 0.097;
		break;
		case 60:
		multiplier = 0.12;
		break;
		case 70:
		multiplier = 0.147;
		break;
		case 73:
		multiplier = 0.152;
		break;
		case 81:
		multiplier = 0.155;
		break;
		case 90:
		multiplier = 0.157;
		break;
		case 100:
		multiplier = 0.183;
		break;
		case 110:
		multiplier = 0.188;
		break;
		case 120:
		multiplier = 0.198;
		break;
		case 135:
		multiplier = 0.216;
		break;
		case 137:
		multiplier = 0.229;
		break;
		case 146:
		multiplier = 0.234;
		break;
		case 165:
		multiplier = 0.241;
		break;
		case 170:
		multiplier = 0.25;
		break;
		case 183:
		multiplier = 0.289;
		break;
		case 192:
		multiplier = 0.33;
		break;
		case 208:
		multiplier = 0.356;
		break;
		case 220:
		multiplier = 0.38;
		break;
		case 267:
		multiplier = 0.445;
		break;		 
		 default:
		 alert("undefined paper weight!");
		 exit(-2);
		 break;
		}
	
	width = multiplier *numberOfPages/2;
	width+= modifiedSize;
	
/*	var minWSpine = 0.353;
	if (width < minWSpine) {
		alert("Minimum width of spine is: "+minWSpine);
		width = minWSpine;
		}
		*/
	
	return width;
	
	}




function makeDocument( pageHeight, pageWidth, name) {
	var preset = app.documentPresets.add();
	//preset.name = name;
	preset.facingPages =  true;
	//preset.allowPageShuffle = false;
	preset.pageHeight = pageHeight;
	preset.pageWidth = pageWidth;
	preset.documentBleedUniformSize = true;
	preset.documentBleedTopOffset = "3mm";	
	preset.columnCount = 1;
	preset.columnGutter = "0";
	preset.bottom = "0p"
	preset.left = "0p"
	preset.right = "0p"
	preset.top = "0p"

	
	var newDocument = app.documents.add(true,preset);
	
	preset.remove();
	
	return newDocument;
	
	}

//creates a new document and puts the cover in the first spread sheet
function createCover(horizontalUnits, verticalUnits, pageHeight, pageWidth, spineWidth, flapsWidth, cover) {


	app.viewPreferences.horizontalMeasurementUnits = horizontalUnits;
	app.viewPreferences.verticalMeasurementUnits = verticalUnits;
	
	var newDocument = makeDocument(pageHeight, pageWidth, "CoverBookPreset_1");
	newDocument.viewPreferences.horizontalMeasurementUnits = horizontalUnits;
	newDocument.viewPreferences.verticalMeasurementUnits = verticalUnits;
	
	newDocument.documentPreferences.allowPageShuffle = false;
	
	
	var ppi = newDocument.viewPreferences.pointsPerInch;
	
	var pointsH = convertUnitsToPoints(verticalUnits, pageHeight,ppi);
	var pointsW = convertUnitsToPoints(horizontalUnits, pageWidth,ppi);
	var pointsWSpine = convertUnitsToPoints(MeasurementUnits.MILLIMETERS, spineWidth,ppi);
	
	switch(cover) {
		case 0:
		case 1:
		pointsWSpine+= convertUnitsToPoints(MeasurementUnits.MILLIMETERS, 0.5,ppi);
		break;
		case 2:
		pointsWSpine+= convertUnitsToPoints(MeasurementUnits.MILLIMETERS, 5,ppi);
		pointsW+= convertUnitsToPoints(MeasurementUnits.MILLIMETERS, 5,ppi);
		pointsH+= convertUnitsToPoints(MeasurementUnits.MILLIMETERS, 10,ppi);
		break;
		case 3:
		pointsWSpine+= convertUnitsToPoints(MeasurementUnits.MILLIMETERS, 5,ppi);
		pointsW+= convertUnitsToPoints(MeasurementUnits.MILLIMETERS,3,ppi);
		break;
		
	}
	
	if (pointsWSpine < convertUnitsToPoints(MeasurementUnits.MILLIMETERS, 0.353,ppi)) {
		alert("Error! Book spine is too thin! Required width is at least 0.353 mm");
		newDocument.close(SaveOptions.NO);
		exit(-1);
	
	}
		
	
		
	var spread = newDocument.spreads[0];
	
	var leftPage = newDocument.pages.add();
	var spine = newDocument.pages.add();
	var rightPage = newDocument.pages.add();
	
	
	leftPage.resize(pointsW,pointsH);
	spine.resize(pointsWSpine,pointsH);
	rightPage.resize(pointsW,pointsH);
	
	leftPage.move(LocationOptions.AT_END, spread);
	spine.move(LocationOptions.AT_END, spread);
	rightPage.move(LocationOptions.AT_END, spread);
	
	
	newDocument.pages[0].remove();
	
	if (flapsWidth > 0 && (cover == 1 | cover == 3)) {
		var pointsFlapWidth = convertUnitsToPoints(horizontalUnits, flapsWidth,ppi);
		var leftFlap = newDocument.pages.add();
		var rightFlap = newDocument.pages.add();
		
		leftFlap.resize(pointsFlapWidth,pointsH);
		rightFlap.resize(pointsFlapWidth,pointsH);
		
		leftFlap.move(LocationOptions.AT_BEGINNING, spread);
		rightFlap.move(LocationOptions.AT_END, spread);
		
		}
	
	}

//converts any units to points
function convertUnitsToPoints(_unitType, _num,_ppi) {
	
	switch(_unitType) {
		case MeasurementUnits.AGATES:
			return _num*_ppi/14;
			break;
		case MeasurementUnits.CENTIMETERS:
			return _num*_ppi/2.54;
			break;
		case MeasurementUnits.CICEROS:
			return _num*12;
			break;
		case MeasurementUnits.CUSTOM:
			alert("Cannot convert from custom measurement units");
			break;
		case MeasurementUnits.INCHES:
			return  _num*_ppi;
			break;
		case MeasurementUnits.INCHES_DECIMAL:
			return _num*_ppi
			break;
		case MeasurementUnits.MILLIMETERS:
			return _num*_ppi/25.4;
			break;
		case MeasurementUnits.PICAS:
			return _num*12;
			break;
		case MeasurementUnits.POINTS:
			return _num;
			break;
		default:
			alert("error, undefined type of measurement units");
			break;
		}
	
		
	}